PROGRAM ngi_ionogram
!
! Fortran90 program to convert VIPIR NetCDF format gridded ionogram files
! .NGI into text file ionogram formats for gnuplot plotting
!
!  09Sep11
!  T. Bullett   CIRES
!
IMPLICIT NONE
!
INCLUDE 'sct.f90'
INCLUDE 'ionogram.f90'
!
!
TYPE(IonogramType):: Ionogram
REAL,PARAMETER :: VERSION=0.10 
INTEGER :: IRNG,IFRQ
! MAXFRQ and MAXRNG defined in ionogram.f90
REAL:: FBANDL = 0.0, FBANDH = 30000.0

! 
! For command line parsing
INTEGER :: K,NA
LOGICAL :: SKIP=.FALSE., INFOK=.FALSE., GOTFILE=.FALSE. 
LOGICAL :: IGRAM,BLANKF
CHARACTER(120) :: INFILE,CLP
!
! Read the command line for arguments
BLANKF=.FALSE.
K = IARGC() 
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: ngi_ionogram filename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Convert VIPIR netCDF ionograms into text format for gnuplot'
      WRITE(*,*)' '
      WRITE(*,*)' Filename must be provided on the command line.'
      WRITE(*,*)' Options:'
      WRITE(*,*)' -f1 Lower frequency select band [kHz]'
      WRITE(*,*)' -f2 Upper frequency select band [kHz]'
      WRITE(*,*)' -bf Blank between each Frequency'
      WRITE(*,*)' '
      RETURN
   ELSE IF (SKIP) THEN
      SKIP=.FALSE.
   ELSE IF (CLP.EQ.'-f1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) FBANDL
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-f2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) FBANDH
      SKIP=.TRUE.
   ELSE IF  (CLP.EQ.'-bf') THEN
      BLANKF=.TRUE.
   ELSE
!    This is the input file name
     GOTFILE=.TRUE.
     INFILE=CLP
   ENDIF
ENDDO

IF (.NOT.GOTFILE) THEN
   WRITE(0,*) "ngi_ionogram - error - No input file specified"
   CALL EXIT(-1)
ENDIF
!
!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(0,*) ' ngi2rdf - error - Missing input file'
   CALL EXIT(-1)
ENDIF

CALL Read_netCDF3(INFILE,   Ionogram)
IGRAM=.TRUE.
IF (IGRAM) THEN
  DO IFRQ=1, Ionogram%NFREQ
      WRITE(*,102) 'IFRQ','IRNG','UT_Time','dt','Freq','Range','TPower','TSNR','OPower','OSNR','XPower','XSNR'
      IF( (Ionogram%Frequency(IFRQ).LT.FBANDL).OR.(Ionogram%Frequency(IFRQ).GT.FBANDH) ) CYCLE
      DO IRNG=1,Ionogram%NRANGE
         WRITE(*,103) IFRQ, IRNG, Ionogram%Time(IFRQ), Ionogram%Time(IFRQ)-Ionogram%Time(1),&
         Ionogram%Frequency(IFRQ)/1000.0, Ionogram%Range(IRNG), & 
         Ionogram%TPower(IRNG,IFRQ), Ionogram%TPower(IRNG,IFRQ)-Ionogram%TNoiseLevel(IFRQ), &
         Ionogram%OPower(IRNG,IFRQ), Ionogram%OPower(IRNG,IFRQ)-Ionogram%ONoiseLevel(IFRQ), &
         Ionogram%XPower(IRNG,IFRQ), Ionogram%XPower(IRNG,IFRQ)-Ionogram%XNoiseLevel(IFRQ)
      ENDDO
      IF(BLANKF) WRITE(*,*)
   ENDDO
ENDIF


102 FORMAT ('#',A7,A8,1X,4A12,A8,6A8)
103 FORMAT (2I8,1X,4F12.5, F8.2, 6F8.2)

END PROGRAM ngi_ionogram

