SUBROUTINE Read_netCDF3(NETCDFILE,   Ionogram)
!
! Program to read a NetCDF format data file which holds
! ionogram data computed from Raw ionogram data (RIQ for VIPIR, MMM,SBF,RSF from Digisondes, etc)
! 
! This program reads the netCDF file using Version 3.6 standards.
!
!
!   Terry Bullett
!   University of Colorado
!   30 March 2009 
!
! License:  Public Domain
!
! "To glorify God by exploring His universe with reverence"
!
! 06Sep11 TWB  Modified the read routine to match the Sep 2010 changes to the write routine
! 

USE netcdf
IMPLICIT NONE
INCLUDE 'sct.f90' 
INCLUDE 'ionogram.f90'
!
CHARACTER(LEN=120),INTENT(IN)::NETCDFILE 
TYPE(IonogramType),INTENT(OUT):: Ionogram
!
! Local Variables
INTEGER:: I,IERR,NRANGE,NFREQ,NDOPP,SCTsize
INTEGER(KIND=1),DIMENSION(100000):: ByteSCT
REAL,PARAMETER:: Version=1.21
CHARACTER(20) :: JUNK 
REAL,DIMENSION(2,1024) :: IQARRAY
!
INTEGER,DIMENSION(2) :: IonogramArray,BasePairArray,BaseDiffsArray,RxArray,Char032Array,DriveArray,BaudArray,HasValues
INTEGER,DIMENSION(3) :: WordFormatValues
INTEGER,DIMENSION(4) :: CoherenceArray,DopplerArray
INTEGER,DIMENSION(4) :: TuneTypeValues 
INTEGER,DIMENSION(5) :: RfControlValues 
INTEGER:: YearID,DaynumberID,MonthID,DayID,HourID,MinuteID,SecondID,EpochID,LatitudeID,LongitudeID,AltitudeID
INTEGER:: ncid,Range_sizeID,Frequency_sizeID,Doppler_sizeID,RangeValID,FrequencyValID,TimeValID,SCTversionID
INTEGER:: DIM002ID,DIM003ID,NumRxPairID,DIM008ID,DIM016ID,DIM032ID,DIM064ID,DIM128ID
INTEGER:: DIM160ID,DIM256ID,DIM512ID,DIM1024ID,DIM2048ID,DIM8192ID
INTEGER:: TxDriveID,Range0ID,RangeGateOffsetID,URSIID,StationNameID,LinearStepID,LogStepID,NumAveID
INTEGER:: PRIID,GateCountID,GateStartID,GateEndID,GateStepID,FreqStartID,FreqEndID,FreqStepsID,TuneTypeID
INTEGER:: MagLatID,MagLonID,MagDipID,GyroFreqID 
INTEGER:: HasTotalPowerID,TotalPowerID,TotalNoiseID
INTEGER:: HasOModePowerID,OModePowerID,OModeNoiseID
INTEGER:: HasXModePowerID,XModePowerID,XModeNoiseID
INTEGER:: HasVLoSID,VLoSID
INTEGER:: HasDopplerID,DopplerID
INTEGER:: HasSPGRID,SPGRID
INTEGER:: HasZenithID,ZenithID
INTEGER:: HasAzimuthID,AzimuthID
INTEGER:: HasCoherenceID,BaselinePairsID,BaselineDiffsID,CoherenceID 
!! 
! For VIPIR
INTEGER:: SCTsizeID,PCTsizeID,PRIsizeID,SCTID,MagicID,SCTReadmeID,SCTUserID,DecimationMethodID,DecimationThresholdID
! SCT%station
INTEGER:: StationFileIdID,RxNameID,RxLatitudeID,RxLongitudeID,RxAltitudeID,RxAntennaTypeID,RxCountID
INTEGER:: RxPositionID,RxDirectionID,RxHeightID,RxCableLengthID, FrontendAttenID
INTEGER:: TxNameID,TxLatitudeID,TxLongitudeID,TxAltitudeID,TxAntennaTypeID,TxVectorID,TxHeightID,TxCableLengthID
INTEGER:: DriveBandCountID,DriveBandBoundsID,DriveBandAttenID,RfControlID,RefTypeID,ClockTypeID,StationUserID
! SCT%timing
INTEGER:: TimingFileIdID,PriCountID,IonogramCountID,HoldoffID,DataWidthID
INTEGER:: DataStartID,DataBaudCountID,DataWaveFileIdID,DataBaudID,DataPairsID,CalStartID
INTEGER:: CalWidthID,CalBaudCountID,CalWaveFileIdID,CalBaudID,CalPairsID,TimingUserID
! SCT%frequency
INTEGER:: FrequencyFileIdID,BaseStepsID,BaseTableID,FrequencyTableIdID,TuneStepsID,PulseCountID,PulsePatternID,PulseOffsetID
INTEGER:: RampStepsID,RampRepeatsID,DriveTableID,FrequencyUserID
! SCT%receiver
INTEGER:: ReceiverFileIdID,RxChanID,RxMapID,WordFormatID,RxCic2DecID,RxCic2InterpID,RxCic2ScaleID,RxCic5DecID,RxCic5ScaleID
INTEGER:: RxRcf_TypeID,RxRcfDecID,RxRcfTapsID,RxCoefficientsID,RxAnalogDelayID,ReceiverUserID
! SCT%receiver
INTEGER:: ExciterFileIdID,ExCic2ScaleID,ExCic2DecID,ExCic2InterpID,ExCic5InterpID,ExRcfTypeID,ExRcfTapsID,ExRcfTapsPhaseID
INTEGER:: ExCoefficientsID,ExAnalogDelayID,ExciterUserID
! SCT%monitor
INTEGER:: BalunCurrentsID,BalunStatusID,FrontendStatusID,ReceiverStatusID,ExciterStatusID,MonitorUserID
!
!
! For Digisondes
INTEGER:: PREFACEID, PREFsizeID
INTEGER:: DigisondePrefLen=77
!
!
CALL ZeroSCT(Ionogram%SCT)
!
! Open the file for reading and get it's "ncid" value
CALL CHECK( nf90_open(NETCDFILE,NF90_NOWRITE,  ncid) )
!
! Read the dimensions
! RANGE, FREQUENCY, DOPPLER
!
! Unlike the write routine, which must first define the file before 
! data can be written, some reads and inquiries depend on the contents
! of other data, especially the Has_xxxx  flags.  So we can't just get
! all of the ID's in one section of the code and read them all in another.
! This makes software maintenance between the read and write versions harder.
!
CALL CHECK( nf90_inq_dimid(ncid, "NumRange",   Range_sizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, Range_sizeID, JUNK, NRANGE) )
CALL CHECK( nf90_inq_dimid(ncid, "NumFrequency", Frequency_sizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, Frequency_sizeID,  JUNK, NFREQ ) )
CALL CHECK( nf90_inq_dimid(ncid, "NumDoppler", Doppler_sizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, Doppler_sizeID,  JUNK, NDOPP ) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM002",    DIM002ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM003",    DIM003ID) )
CALL CHECK( nf90_inq_dimid(ncid, "NumRxPair", NumRxPairID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM008",    DIM008ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM016",    DIM016ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM032",    DIM032ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM064",    DIM064ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM128",    DIM128ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM160",    DIM160ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM256",    DIM256ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM512",    DIM512ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM1024",   DIM1024ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM2048",   DIM2048ID) )
CALL CHECK( nf90_inq_dimid(ncid, "DIM8192",   DIM8192ID) )
!
! RANGE, FREQUENCY, DOPPLER
Ionogram%NRANGE=NRANGE
Ionogram%NFREQ=NFREQ
Ionogram%NDOPP=NDOPP
!
! Define Array Dimensions.
IonogramArray= (/ Range_sizeID, Frequency_sizeID/)
CoherenceArray= (/ DIM002ID, NumRxPairID, Range_sizeID, Frequency_sizeID /)
DopplerArray= (/ DIM002ID, Doppler_sizeID, Range_sizeID, Frequency_sizeID /)
BasePairArray= (/ DIM002ID, NumRxPairID /)
BaseDiffsArray= (/ DIM003ID, NumRxPairID /)
RxArray= (/ DIM003ID, DIM032ID /)
Char032Array= (/ DIM032ID, DIM032ID /)
DriveArray= (/ DIM002ID, DIM064ID /)
BaudArray= (/ DIM002ID, DIM1024ID /)
HasValues= (/ 0, 1/)
RfControlValues= (/ -1, 0, 2, 2, 3 /)
TuneTypeValues= (/ 1, 2, 3, 4 /)
WordFormatValues= (/ 1, 2, 3 /)
!
! SCT for VIPIR sounders
CALL CHECK( nf90_inq_dimid(ncid, "SCTsize",   SCTsizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, SCTsizeID, JUNK, Ionogram%SCT%sounding_table_size) )
CALL CHECK( nf90_inq_dimid(ncid, "PCTsize",   PCTsizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, PCTsizeID, JUNK, Ionogram%SCT%pulse_table_size) )
CALL CHECK( nf90_inq_dimid(ncid, "PRIsize",   PRIsizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, PRIsizeID, JUNK,Ionogram%SCT%raw_data_size ) )
! Preface for Digisondes
CALL CHECK( nf90_inq_dimid(ncid, "PREFACEsize", PREFsizeID) )
   CALL CHECK( nf90_inquire_dimension(ncid, PREFsizeID, JUNK, DigisondePrefLen  ) )
!
! Other brands & Models of ionosondes setting information go here
!
! Read the Variables
! 
! Station
CALL CHECK( nf90_inq_varid(ncid,"URSI",         URSIID)            )
  CALL CHECK( nf90_get_var(ncid,URSIID,         Ionogram%URSICODE))
CALL CHECK( nf90_inq_varid(ncid,"StationName",  StationNameID)     )
  CALL CHECK( nf90_get_var(ncid,StationNameID,  Ionogram%Station)  )
! Time
CALL CHECK( nf90_inq_varid(ncid,"year",         YearID)            )
  CALL CHECK( nf90_get_var(ncid,YearID,         Ionogram%Year)     )
CALL CHECK( nf90_inq_varid(ncid,"daynumber",    DaynumberID)       )
  CALL CHECK( nf90_get_var(ncid,DaynumberID,    Ionogram%DayNumber))
CALL CHECK( nf90_inq_varid(ncid,"month",        MonthID)           )
  CALL CHECK( nf90_get_var(ncid,MonthID,        Ionogram%Month)    )
CALL CHECK( nf90_inq_varid(ncid,"day",          DayID)             )
  CALL CHECK( nf90_get_var(ncid,DayID,          Ionogram%Day)      )
CALL CHECK( nf90_inq_varid(ncid,"hour",         HourID)            )
  CALL CHECK( nf90_get_var(ncid,HourID,         Ionogram%Hour)     )
CALL CHECK( nf90_inq_varid(ncid,"minute",       MinuteID)          )
  CALL CHECK( nf90_get_var(ncid,MinuteID,       Ionogram%Minute)   )
CALL CHECK( nf90_inq_varid(ncid,"second",       SecondID)          )
  CALL CHECK( nf90_get_var(ncid,SecondID,       Ionogram%Second)   )
CALL CHECK( nf90_inq_varid(ncid,"epoch",        EpochID)           )
  CALL CHECK( nf90_get_var(ncid,EpochID,        Ionogram%EpochTime))
! Sensor Location
CALL CHECK( nf90_inq_varid(ncid,"latitude",     LatitudeID)        )
  CALL CHECK( nf90_get_var(ncid,LatitudeID,     Ionogram%Latitude) )
CALL CHECK( nf90_inq_varid(ncid,"longitude",    LongitudeID)       )
  CALL CHECK( nf90_get_var(ncid,LongitudeID,    Ionogram%Longitude))
CALL CHECK( nf90_inq_varid(ncid,"altitude",     AltitudeID)        )
  CALL CHECK( nf90_get_var(ncid,AltitudeID,     Ionogram%Altitude) )
! Magnetic Field
CALL CHECK( nf90_inq_varid(ncid,"MagLat",       MagLatID)          )
  CALL CHECK( nf90_get_var(ncid,MagLatID,       Ionogram%MagLat)   )
CALL CHECK( nf90_inq_varid(ncid,"MagLon",       MagLonID)          )
  CALL CHECK( nf90_get_var(ncid,MagLonID,       Ionogram%MagLon)   )
CALL CHECK( nf90_inq_varid(ncid,"MagDip",       MagDipID)          )
  CALL CHECK( nf90_get_var(ncid,MagDipID,       Ionogram%MagDip)   )
CALL CHECK( nf90_inq_varid(ncid,"GyroFreq",     GyroFreqID)        )
  CALL CHECK( nf90_get_var(ncid,GyroFreqID,     Ionogram%GyroFreq) )
! Range Settings  
CALL CHECK( nf90_inq_varid(ncid,"pri",          PRIID)             )
  CALL CHECK( nf90_get_var(ncid,PRIID,         Ionogram%PRI)       )
CALL CHECK( nf90_inq_varid(ncid,"range_gate_offset", RangeGateOffsetID) )
  CALL CHECK( nf90_get_var(ncid,RangeGateOffsetID, Ionogram%RangeGateOffset) )
CALL CHECK( nf90_inq_varid(ncid,"gate_count",  GateCountID)        )
  CALL CHECK( nf90_get_var(ncid,GateCountID,   Ionogram%GateCount) )
CALL CHECK( nf90_inq_varid(ncid,"gate_start",  GateStartID)        )
  CALL CHECK( nf90_get_var(ncid,GateStartID,   Ionogram%GateStart) )
CALL CHECK( nf90_inq_varid(ncid,"gate_end",    GateEndID)          )
  CALL CHECK( nf90_get_var(ncid,GateEndID,     Ionogram%GateEnd)   )
CALL CHECK( nf90_inq_varid(ncid,"gate_step",   GateStepID)         )
  CALL CHECK( nf90_get_var(ncid,GateStepID,    Ionogram%GateStep)  )
CALL CHECK( nf90_inq_varid(ncid,"Range0",      Range0ID)           )
  CALL CHECK( nf90_get_var(ncid,Range0ID,      Ionogram%RANGE0)    )
! Range Array
CALL CHECK( nf90_inq_dimid(ncid,"NumRange",    Range_sizeID)                 )
  CALL CHECK( nf90_inquire_dimension(ncid,Range_sizeID, JUNK,Ionogram%NRANGE))
  NRANGE=Ionogram%NRANGE
CALL CHECK( nf90_inq_varid(ncid,"Range",       RangeValID)                   )
  CALL CHECK( nf90_get_var(ncid,RangeValID,    Ionogram%Range(1:NRANGE))     )
!
! Frequency Settings
CALL CHECK( nf90_inq_varid(ncid,"freq_start",  FreqStartID)        )
  CALL CHECK( nf90_get_var(ncid,FreqStartID,   Ionogram%FreqStart) )
CALL CHECK( nf90_inq_varid(ncid,"freq_end",    FreqEndID)          )
  CALL CHECK( nf90_get_var(ncid,FreqEndID,     Ionogram%FreqEnd)   )
CALL CHECK( nf90_inq_varid(ncid,"tune_type",   TuneTypeID)         )
  CALL CHECK( nf90_get_var(ncid,TuneTypeID,    Ionogram%TuneType)  )
CALL CHECK( nf90_inq_varid(ncid,"freq_count",  FreqStepsID)        )
  CALL CHECK( nf90_get_var(ncid,FreqStepsID,   Ionogram%FreqSteps) )
CALL CHECK( nf90_inq_varid(ncid,"linear_step", LinearStepID)       )
  CALL CHECK( nf90_get_var(ncid,LinearStepID,  Ionogram%LinearStep))
CALL CHECK( nf90_inq_varid(ncid,"log_step",    LogStepID)          )
  CALL CHECK( nf90_get_var(ncid,LogStepID,     Ionogram%LogStep)   )
! Freqeuncy Array
CALL CHECK( nf90_inq_dimid(ncid,"NumFrequency", Frequency_sizeID)                 )
  CALL CHECK( nf90_inquire_dimension(ncid,Frequency_sizeID, JUNK,Ionogram%NFREQ)  )
  NFREQ=Ionogram%NFREQ
CALL CHECK( nf90_inq_varid(ncid,"Frequency",   FrequencyValID)                    ) 
  CALL CHECK( nf90_get_var(ncid,FrequencyValID, Ionogram%Frequency(1:NFREQ))      )
!
! Following arrays are same size as Frequency
! Time Array
CALL CHECK( nf90_inq_varid(ncid,"Time",        TimeValID)                )
  CALL CHECK( nf90_get_var(ncid,TimeValID,     Ionogram%Time(1:NFREQ))   )
!
! Transmitter Drive Level
CALL CHECK( nf90_inq_varid(ncid,"TxDrive",     TxDriveID)                )
  CALL CHECK( nf90_get_var(ncid,TxDriveID,     Ionogram%TxDrive(1:NFREQ)))
!
! Normalization Factor
CALL CHECK( nf90_inq_varid(ncid,"NumAve",      NumAveID)                 )
CALL CHECK( nf90_get_var(ncid,NumAveID,        Ionogram%NumAve(1:NFREQ)) )
!
! Read the VIPIR SCT ID's into NetCDF variables
! Note: the redundancies in the write_netcdf function are not redundant here
! due to replications in the Ionogram() and SCT() structures.
! Re-defining the ID values is made for symmetry
CALL CHECK( nf90_inq_varid(ncid, "Magic",   MagicID) ) 
    CALL CHECK( nf90_get_var(ncid,MagicID,               Ionogram%SCT%magic )  )
CALL CHECK( nf90_inq_varid(ncid, "SCT_version", SCTversionID)                  )
   CALL CHECK( nf90_get_var(ncid,SCTversionID, Ionogram%SCT%struct_version )   )
CALL CHECK( nf90_inq_varid(ncid, "SCT_version", SCTversionID)                  )
   CALL CHECK( nf90_get_var(ncid,SCTversionID, Ionogram%SCT%struct_version )   )
! Elements in the Ionogram strucutre not duplicated into NetCDF values
Ionogram%SCT%start_year = Ionogram%Year
Ionogram%SCT%start_daynumber = Ionogram%DayNumber
Ionogram%SCT%start_month = Ionogram%Month
Ionogram%SCT%start_day = Ionogram%Day
Ionogram%SCT%start_hour = Ionogram%Hour
Ionogram%SCT%start_minute = Ionogram%Minute
Ionogram%SCT%start_second = Ionogram%Second
Ionogram%SCT%start_epoch = Ionogram%EpochTime
CALL CHECK( nf90_inq_varid(ncid,"SCT_readme",  SCTReadmeID) )
   CALL CHECK( nf90_get_var(ncid,SCTReadmeID,  Ionogram%SCT%readme )  )
CALL CHECK( nf90_inq_varid(ncid,"decimation_method", DecimationMethodID) )
   CALL CHECK( nf90_get_var(ncid,DecimationMethodID,    Ionogram%SCT%decimation_method ) )
CALL CHECK( nf90_inq_varid(ncid, "decimation_threshold", DecimationThresholdID) )
   CALL CHECK( nf90_get_var(ncid,DecimationThresholdID, Ionogram%SCT%decimation_threshold ) )
CALL CHECK( nf90_inq_varid(ncid,"SCT_user",    SCTUserID ) )
   CALL CHECK( nf90_get_var(ncid,SCTUserID,    Ionogram%SCT%user ) )
!
! SCT%station
! SCT Receiver Station Info
CALL CHECK( nf90_inq_varid(ncid, "station_file_id",   StationFileIdID) )
   CALL CHECK( nf90_get_var(ncid,StationFileIdID,     Ionogram%SCT%station%file_id) )
Ionogram%SCT%station%ursi_id=Ionogram%URSICODE
CALL CHECK( nf90_inq_varid(ncid, "rx_name",      RxNameID) )
   CALL CHECK( nf90_get_var(ncid,RxNameID,       Ionogram%SCT%station%rx_name) )
CALL CHECK( nf90_inq_varid(ncid,"rx_latitude",   RxLatitudeID) )
   CALL CHECK( nf90_get_var(ncid,RxLatitudeID,   Ionogram%SCT%station%rx_latitude) )
CALL CHECK( nf90_inq_varid(ncid,"rx_longitude",  RxLongitudeID) )
   CALL CHECK( nf90_get_var(ncid,RxLongitudeID,  Ionogram%SCT%station%rx_longitude) )
CALL CHECK( nf90_inq_varid(ncid,"rx_altitude",   RxAltitudeID) )
   CALL CHECK( nf90_get_var(ncid,RxAltitudeID,   Ionogram%SCT%station%rx_altitude) )
CALL CHECK( nf90_inq_varid(ncid,"rx_count",      RxCountID) )
   CALL CHECK( nf90_get_var(ncid,RxCountID,         Ionogram%SCT%station%rx_count) )
CALL CHECK( nf90_inq_varid(ncid, "rx_antenna_type", RxAntennaTypeID) )
   CALL CHECK( nf90_get_var(ncid,RxAntennaTypeID,   Ionogram%SCT%station%rx_antenna_type) )
CALL CHECK( nf90_inq_varid(ncid, "rx_position",     RxPositionID) )
   CALL CHECK( nf90_get_var(ncid,RxPositionID,      Ionogram%SCT%station%rx_position) )
CALL CHECK( nf90_inq_varid(ncid, "rx_direction",    RxDirectionID) )
   CALL CHECK( nf90_get_var(ncid,RxDirectionID,     Ionogram%SCT%station%rx_direction) )
CALL CHECK( nf90_inq_varid(ncid, "rx_height",       RxHeightID) )
   CALL CHECK( nf90_get_var(ncid,RxHeightID,        Ionogram%SCT%station%rx_height) )
CALL CHECK( nf90_inq_varid(ncid, "rx_cable_length", RxCableLengthID) )
   CALL CHECK( nf90_get_var(ncid,RxCableLengthID,   Ionogram%SCT%station%rx_cable_length) )
! Attenuation
CALL CHECK( nf90_inq_varid(ncid,"frontend_atten",   FrontendAttenID) )
   CALL CHECK( nf90_get_var(ncid,FrontendAttenID,   Ionogram%SCT%station%frontend_atten) )
! Transmitter
CALL CHECK( nf90_inq_varid(ncid, "tx_name",         TxNameID) )
   CALL CHECK( nf90_get_var(ncid,TxNameID,          Ionogram%SCT%station%tx_name) )
CALL CHECK( nf90_inq_varid(ncid,"tx_latitude",      TxLatitudeID) )
   CALL CHECK( nf90_get_var(ncid,TxLatitudeID,      Ionogram%SCT%station%tx_latitude) )
CALL CHECK( nf90_inq_varid(ncid,"tx_longitude",     TxLongitudeID) )
   CALL CHECK( nf90_get_var(ncid,TxLongitudeID,     Ionogram%SCT%station%tx_longitude) )
CALL CHECK( nf90_inq_varid(ncid,"tx_altitude",      TxAltitudeID) )
   CALL CHECK( nf90_get_var(ncid,TxAltitudeID,      Ionogram%SCT%station%tx_altitude) )
CALL CHECK( nf90_inq_varid(ncid, "tx_antenna_type", TxAntennaTypeID) )
   CALL CHECK( nf90_get_var(ncid,TxAntennaTypeID,   Ionogram%SCT%station%tx_antenna_type) )
CALL CHECK( nf90_inq_varid(ncid, "tx_vector",       TxVectorID) )
   CALL CHECK( nf90_get_var(ncid,TxVectorID,        Ionogram%SCT%station%tx_vector) )
CALL CHECK( nf90_inq_varid(ncid, "tx_height",       TxHeightID) )
   CALL CHECK( nf90_get_var(ncid,TxHeightID,        Ionogram%SCT%station%tx_height) )
CALL CHECK( nf90_inq_varid(ncid, "tx_cable_length",  txCableLengthID) )
   CALL CHECK( nf90_get_var(ncid,TxCableLengthID,    Ionogram%SCT%station%tx_cable_length) )
! Drive Bands
CALL CHECK( nf90_inq_varid(ncid,"drive_band_count",  DriveBandCountID) )
   CALL CHECK( nf90_get_var(ncid,DriveBandCountID,   Ionogram%SCT%station%drive_band_count) )
CALL CHECK( nf90_inq_varid(ncid,"drive_band_bounds", DriveBandBoundsID) )
   CALL CHECK( nf90_get_var(ncid,DriveBandBoundsID,  Ionogram%SCT%station%drive_band_bounds) )
CALL CHECK( nf90_inq_varid(ncid,"drive_band_atten",  DriveBandAttenID) )
   CALL CHECK( nf90_get_var(ncid,DriveBandAttenID,   Ionogram%SCT%station%drive_band_atten) )
CALL CHECK( nf90_inq_varid(ncid,"rf_control",        RfControlID) )
   CALL CHECK( nf90_get_var(ncid,RfControlID,        Ionogram%SCT%station%rf_control) )
CALL CHECK( nf90_inq_varid(ncid, "ref_type",         RefTypeID) )
   CALL CHECK( nf90_get_var(ncid,RefTypeID,          Ionogram%SCT%station%ref_type) )
CALL CHECK( nf90_inq_varid(ncid, "clock_type",       ClockTypeID) )
   CALL CHECK( nf90_get_var(ncid,ClockTypeID,        Ionogram%SCT%station%clock_type) )
CALL CHECK( nf90_inq_varid(ncid, "station_user",     StationUserID) )
   CALL CHECK( nf90_get_var(ncid,StationUserID,      Ionogram%SCT%station%user) )
! Timing
CALL CHECK( nf90_inq_varid(ncid, "timing_file_id", TimingFileIdID) )
   CALL CHECK( nf90_get_var(ncid,TimingFileIdID,   Ionogram%SCT%timing%file_id) )
CALL CHECK( nf90_inq_varid(ncid,"pri",             PRIID)             )
   CALL CHECK( nf90_get_var(ncid,PriID,            Ionogram%SCT%timing%pri) )
CALL CHECK( nf90_inq_varid(ncid,"pri_count",       PriCountID) )
   CALL CHECK( nf90_get_var(ncid,PriCountID,       Ionogram%SCT%timing%pri_count) )
CALL CHECK( nf90_inq_varid(ncid,"ionogram_count",  IonogramCountID) )
   CALL CHECK( nf90_get_var(ncid,IonogramCountID,  Ionogram%SCT%timing%ionogram_count) )
CALL CHECK( nf90_inq_varid(ncid,"holdoff",         HoldoffID) )
   CALL CHECK( nf90_get_var(ncid,HoldoffID,        Ionogram%SCT%timing%holdoff) )
CALL CHECK( nf90_inq_varid(ncid,"range_gate_offset", RangeGateOffsetID) )
   CALL CHECK( nf90_get_var(ncid,RangeGateOffsetID,  Ionogram%SCT%timing%range_gate_offset) )
CALL CHECK( nf90_inq_varid(ncid,"gate_count",        GateCountID) )
   CALL CHECK( nf90_get_var(ncid,GateCountID,        Ionogram%SCT%timing%gate_count) )
CALL CHECK( nf90_inq_varid(ncid,"gate_start",        GateStartID) )
   CALL CHECK( nf90_get_var(ncid,GateStartID,        Ionogram%SCT%timing%gate_start) )
CALL CHECK( nf90_inq_varid(ncid,"gate_end",          GateEndID) )
   CALL CHECK( nf90_get_var(ncid,GateEndID,          Ionogram%SCT%timing%gate_end) )
CALL CHECK( nf90_inq_varid(ncid,"gate_step",         GateStepID) )
   CALL CHECK( nf90_get_var(ncid,GateStepID,         Ionogram%SCT%timing%gate_step) )
CALL CHECK( nf90_inq_varid(ncid,"data_start",        DataStartID) )
   CALL CHECK( nf90_get_var(ncid,DataStartID,        Ionogram%SCT%timing%data_start) )
CALL CHECK( nf90_inq_varid(ncid,"date_width",        DataWidthID) )
   CALL CHECK( nf90_get_var(ncid,DataWidthID,        Ionogram%SCT%timing%data_width) )
CALL CHECK( nf90_inq_varid(ncid,"data_baud_count",   DataBaudCountID) )
   CALL CHECK( nf90_get_var(ncid,DataBaudCountID,    Ionogram%SCT%timing%data_baud_count) )
CALL CHECK( nf90_inq_varid(ncid, "data_wave_file",   DataWaveFileIdID) )
   CALL CHECK( nf90_get_var(ncid,DataWaveFileIdID,   Ionogram%SCT%timing%data_wave_file) )
! netCDF cannot handle complex numbers.  Map into a 2 dimensional set of REAL()
CALL CHECK( nf90_inq_varid(ncid,"data_baud",    DataBaudID) )
   CALL CHECK( nf90_get_var(ncid,DataBaudID,    IQARRAY ) )
   DO I=1,1024
      Ionogram%SCT%timing%data_baud(I) = CMPLX( IQARRAY(1,I),IQARRAY(2,I))
   ENDDO
CALL CHECK( nf90_inq_varid(ncid,"data_pairs",   DataPairsID) )
   CALL CHECK( nf90_get_var(ncid,DataPairsID,   Ionogram%SCT%timing%data_pairs) )
CALL CHECK( nf90_inq_varid(ncid,"cal_start",    CalStartID) )
   CALL CHECK( nf90_get_var(ncid,CalStartID,    Ionogram%SCT%timing%cal_start) )
CALL CHECK( nf90_inq_varid(ncid,"cal_width",    CalWidthID) )
   CALL CHECK( nf90_get_var(ncid,CalWidthID,    Ionogram%SCT%timing%cal_width) )
CALL CHECK( nf90_inq_varid(ncid,"cal_baud_count",   CalBaudCountID) )
   CALL CHECK( nf90_get_var(ncid,CalBaudCountID,    Ionogram%SCT%timing%cal_baud_count) )
CALL CHECK( nf90_inq_varid(ncid, "cal_wave_file",   CalWaveFileIdID) )
   CALL CHECK( nf90_get_var(ncid,CalWaveFileIdID,     Ionogram%SCT%timing%cal_wave_file) )
! netCDF cannot handle complex numbers.  Map into a 2 dimensional set of REAL()
CALL CHECK( nf90_inq_varid(ncid,"cal_baud",    CalBaudID) )
   CALL CHECK( nf90_get_var(ncid,CalBaudID,    IQARRAY) )
   DO I=1,1024
      Ionogram%SCT%timing%cal_baud(I) = CMPLX( IQARRAY(1,I),IQARRAY(2,I))
   ENDDO
CALL CHECK( nf90_inq_varid(ncid,"cal_pairs",     CalPairsID) )
   CALL CHECK( nf90_get_var(ncid,CalPairsID,     Ionogram%SCT%timing%cal_pairs) )
CALL CHECK( nf90_inq_varid(ncid, "timing_user",  TimingUserID) )
   CALL CHECK( nf90_get_var(ncid,TimingUserID,   Ionogram%SCT%timing%user) )
! SCT%frequency
CALL CHECK( nf90_inq_varid(ncid, "frequency_file_id",  FrequencyFileIdID) )
   CALL CHECK( nf90_get_var(ncid,FrequencyFileIdID,    Ionogram%SCT%frequency%file_id ) )
CALL CHECK( nf90_inq_varid(ncid,"freq_start",  FreqStartID))
  CALL CHECK( nf90_get_var(ncid,FreqStartID,   Ionogram%SCT%frequency%base_start) )
CALL CHECK( nf90_inq_varid(ncid,"freq_end",    FreqEndID) )
  CALL CHECK( nf90_get_var(ncid,FreqEndID,   Ionogram%SCT%frequency%base_end) )
CALL CHECK( nf90_inq_varid(ncid,"base_steps",  BaseStepsID) )
   CALL CHECK( nf90_get_var(ncid,BaseStepsID,  Ionogram%SCT%frequency%base_steps ) )
CALL CHECK( nf90_inq_varid(ncid,"tune_type",   TuneTypeID))
  CALL CHECK( nf90_get_var(ncid,TuneTypeID,    Ionogram%SCT%frequency%tune_type)  )
CALL CHECK( nf90_inq_varid(ncid,"base_table",  BaseTableID))
   CALL CHECK( nf90_get_var(ncid,BaseTableID,  Ionogram%SCT%frequency%base_table ) )
!  Beware name mismatch and redundancy to SCT%frequency%base_steps.  Legacy. 
CALL CHECK( nf90_inq_varid(ncid,"linear_step", LinearStepID) )
  CALL CHECK( nf90_get_var(ncid,LinearStepID,  Ionogram%SCT%frequency%linear_step))
CALL CHECK( nf90_inq_varid(ncid,"log_step",    LogStepID) )
  CALL CHECK( nf90_get_var(ncid,LogStepID,     Ionogram%SCT%frequency%log_step)   )
CALL CHECK( nf90_inq_varid(ncid, "frequency_table_id", FrequencyTableIdID) )
   CALL CHECK( nf90_get_var(ncid,FrequencyTableIdID,   Ionogram%SCT%frequency%freq_table_id ) )
CALL CHECK( nf90_inq_varid(ncid,"tune_steps",     TuneStepsID) )
   CALL CHECK( nf90_get_var(ncid,TuneStepsID,     Ionogram%SCT%frequency%tune_steps ) )
CALL CHECK( nf90_inq_varid(ncid,"pulse_count",    PulseCountID) )
   CALL CHECK( nf90_get_var(ncid,PulseCountID,    Ionogram%SCT%frequency%pulse_count ) )
CALL CHECK( nf90_inq_varid(ncid,"pulse_pattern",  PulsePatternID) )
   CALL CHECK( nf90_get_var(ncid,PulsePatternID,  Ionogram%SCT%frequency%pulse_pattern ) )
CALL CHECK( nf90_inq_varid(ncid,"pulse_offset",   PulseOffsetID) )
   CALL CHECK( nf90_get_var(ncid,PulseOffsetID,   Ionogram%SCT%frequency%pulse_offset ) )
CALL CHECK( nf90_inq_varid(ncid,"ramp_steps",     RampStepsID) )
   CALL CHECK( nf90_get_var(ncid,RampStepsID,     Ionogram%SCT%frequency%ramp_steps) )
CALL CHECK( nf90_inq_varid(ncid,"ramp_repeats",   RampRepeatsID) )
   CALL CHECK( nf90_get_var(ncid,RampRepeatsID,   Ionogram%SCT%frequency%ramp_repeats ) )
CALL CHECK( nf90_inq_varid(ncid,"drive_table",    DriveTableID) )
   CALL CHECK( nf90_get_var(ncid,DriveTableID,    Ionogram%SCT%frequency%drive_table ) )
CALL CHECK( nf90_inq_varid(ncid, "frequency_user",FrequencyUserID) )
   CALL CHECK( nf90_get_var(ncid,FrequencyUserID, Ionogram%SCT%frequency%user ) )
! SCT%Receiver
CALL CHECK( nf90_inq_varid(ncid, "receiver_file_id",  ReceiverFileIdID) )
   CALL CHECK( nf90_get_var(ncid,ReceiverFileIdID,    Ionogram%SCT%receiver%file_id ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_chan",            RxChanID) )
   CALL CHECK( nf90_get_var(ncid,RxChanID,            Ionogram%SCT%receiver%rx_chan ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_map",             RxMapID) )
   CALL CHECK( nf90_get_var(ncid,RxMapID,             Ionogram%SCT%receiver%rx_map) )
CALL CHECK( nf90_inq_varid(ncid,"word_format",        WordFormatID) )
   CALL CHECK( nf90_get_var(ncid,WordFormatID,        Ionogram%SCT%receiver%word_format ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_cic2_dec",        RxCic2DecID) )
   CALL CHECK( nf90_get_var(ncid,RxCic2DecID,         Ionogram%SCT%receiver%cic2_dec ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_cic2_interp",     RxCic2InterpID) )
   CALL CHECK( nf90_get_var(ncid,RxCic2InterpID,      Ionogram%SCT%receiver%cic2_interp ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_cic2_scale",      RxCic2ScaleID) )
   CALL CHECK( nf90_get_var(ncid,RxCic2ScaleID,       Ionogram%SCT%receiver%cic2_scale ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_cic5_dec",        RxCic5DecID) )
   CALL CHECK( nf90_get_var(ncid,RxCic5DecID,         Ionogram%SCT%receiver%cic5_dec ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_cic5_scale",      RxCic5ScaleID) )
   CALL CHECK( nf90_get_var(ncid,RxCic5ScaleID,       Ionogram%SCT%receiver%cic5_scale ) )
CALL CHECK( nf90_inq_varid(ncid, "rx_rcf_type",       RxRcf_TypeID) )
   CALL CHECK( nf90_get_var(ncid,RxRcf_TypeID,        Ionogram%SCT%receiver%rcf_type ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_rcf_dec",         RxRcfDecID) )
   CALL CHECK( nf90_get_var(ncid,RxRcfDecID,          Ionogram%SCT%receiver%rcf_dec ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_rcf_taps",        RxRcfTapsID) )
   CALL CHECK( nf90_get_var(ncid,RxRcfTapsID,         Ionogram%SCT%receiver%rcf_taps ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_coefficients",    RxCoefficientsID) )
   CALL CHECK( nf90_get_var(ncid,RxCoefficientsID,    Ionogram%SCT%receiver%coefficients ) )
CALL CHECK( nf90_inq_varid(ncid,"rx_analog_delay",    RxAnalogDelayID) )
   CALL CHECK( nf90_get_var(ncid,RxAnalogDelayID,     Ionogram%SCT%receiver%analog_delay ) )
CALL CHECK( nf90_inq_varid(ncid, "receiver_user",     ReceiverUserID) )
   CALL CHECK( nf90_get_var(ncid,ReceiverUserID,      Ionogram%SCT%receiver%user ) )
! SCT%exciter
CALL CHECK( nf90_inq_varid(ncid, "exciter_file_id",   ExciterFileIdID) )
   CALL CHECK( nf90_get_var(ncid,ExciterFileIdID,     Ionogram%SCT%exciter%file_id ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_cic2_scale",       ExCic2ScaleID) )
   CALL CHECK( nf90_get_var(ncid,ExCic2ScaleID,       Ionogram%SCT%exciter%cic_scale ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_cic2_dec",         ExCic2DecID) )
   CALL CHECK( nf90_get_var(ncid,ExCic2DecID,         Ionogram%SCT%exciter%cic2_dec ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_cic2_interp",     ExCic2InterpID) )
   CALL CHECK( nf90_get_var(ncid,ExCic2InterpID,      Ionogram%SCT%exciter%cic2_interp ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_cic5_interp",     ExCic5InterpID) )
   CALL CHECK( nf90_get_var(ncid,ExCic5InterpID,      Ionogram%SCT%exciter%cic5_interp ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_rcf_type",        ExRcfTypeID) )
   CALL CHECK( nf90_get_var(ncid,ExRcfTypeID,         Ionogram%SCT%exciter%rcf_type ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_rcf_taps",        ExRcfTapsID) )
   CALL CHECK( nf90_get_var(ncid,ExRcfTapsID,         Ionogram%SCT%exciter%rcf_taps ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_rcf_taps_phase",  ExRcfTapsPhaseID) )
   CALL CHECK( nf90_get_var(ncid,ExRcfTapsPhaseID,    Ionogram%SCT%exciter%rcf_taps_phase ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_coefficients",    ExCoefficientsID) )
   CALL CHECK( nf90_get_var(ncid,ExCoefficientsID,    Ionogram%SCT%exciter%coefficients ) )
CALL CHECK( nf90_inq_varid(ncid,"ex_analog_delay",    ExAnalogDelayID) )
   CALL CHECK( nf90_get_var(ncid,ExAnalogDelayID,     Ionogram%SCT%exciter%analog_delay ) )
CALL CHECK( nf90_inq_varid(ncid, "exciter_user",      ExciterUserID) )
   CALL CHECK( nf90_get_var(ncid,ExciterUserID,       Ionogram%SCT%exciter%user ) )
! SCT%monitor
CALL CHECK( nf90_inq_varid(ncid,"balun_currents",     BalunCurrentsID) )
   CALL CHECK( nf90_get_var(ncid,BalunCurrentsID,     Ionogram%SCT%monitor%balun_currents ) )
CALL CHECK( nf90_inq_varid(ncid,"balun_status",       BalunStatusID) )
   CALL CHECK( nf90_get_var(ncid,BalunStatusID,       Ionogram%SCT%monitor%balun_status ) )
CALL CHECK( nf90_inq_varid(ncid,"frontend_status",    FrontendStatusID) )
   CALL CHECK( nf90_get_var(ncid,FrontendStatusID,    Ionogram%SCT%monitor%front_end_status ) )
CALL CHECK( nf90_inq_varid(ncid,"receiver_status",    ReceiverStatusID) )
   CALL CHECK( nf90_get_var(ncid,ReceiverStatusID,    Ionogram%SCT%monitor%receiver_status ) )
CALL CHECK( nf90_inq_varid(ncid,"exciter_status",     ExciterStatusID) )
   CALL CHECK( nf90_get_var(ncid,ExciterStatusID,     Ionogram%SCT%monitor%exciter_status ) )
CALL CHECK( nf90_inq_varid(ncid, "monitor_user",      MonitorUserID) )
   CALL CHECK( nf90_get_var(ncid,MonitorUserID,       Ionogram%SCT%monitor%user ) )
!
! Raw SCT for VIPIR sounders: Binary format
CALL CHECK( nf90_inq_dimid(ncid,"SCTsize",  SCTsizeID)                                    )
  CALL CHECK( nf90_inquire_dimension(ncid,SCTsizeID, JUNK,SCTsize))
CALL CHECK( nf90_inq_varid(ncid, "SCT",             SCTID)                            )
   CALL CHECK( nf90_get_var(ncid,SCTID,  ByteSCT(1:Ionogram%SCT%sounding_table_size)) )
! 
! Map the SCT into an array of bytes for the netCDF file.  This is done through a temporary file.
OPEN (UNIT=99, STATUS="SCRATCH",FORM="UNFORMATTED", ACCESS="STREAM")
WRITE(99,IOSTAT=IERR) (ByteSCT(I),I=1,Ionogram%SCT%sounding_table_size)
REWIND(99)
READ(99) Ionogram%SCT2
CLOSE(99)
!
! Preface for Digisondes
CALL CHECK( nf90_inq_dimid(ncid, "PREFACEsize",  PREFsizeID)              )
  CALL CHECK( nf90_inquire_dimension(ncid,PREFsizeID, JUNK,DigisondePrefLen) )
CALL CHECK( nf90_inq_varid(ncid, "PREFACE",            PREFACEID)           )
!
! Other brands & Models go here
!
! Debug
!db CALL FixSCTstrings(Ionogram%SCT)
!db CALL DumpSCT(Ionogram%SCT)
!db write(*,*) "------------------------------------" 
!db CALL FixSCTstrings(Ionogram%SCT2)
!db CALL DumpSCT(Ionogram%SCT2 )
!
! The 2D data arrays.  These are optional, so read them conditionally  
!
! Total Power
CALL CHECK( nf90_inq_varid(ncid, "Has_total_power",  HasTotalPowerID)                   )
   CALL CHECK( nf90_get_var(ncid,HasTotalPowerID,    Ionogram%HasTotalPower)            )
IF (Ionogram%HasTotalPower.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "total_power",   TotalPowerID)                      )
     CALL CHECK( nf90_get_var(ncid,TotalPowerID,     Ionogram%TPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "total_noise",   TotalNoiseID)                      )
     CALL CHECK( nf90_get_var(ncid,TotalNoiseID,     Ionogram%TNoiseLevel(1:NFREQ)    ) )
ENDIF 
!
! Ordinary Mode Power
CALL CHECK( nf90_inq_varid(ncid, "Has_O-mode_power", HasOModePowerID)                   )
  CALL CHECK( nf90_get_var(ncid,HasOModePowerID,     Ionogram%HasOModePower)            )
IF (Ionogram%HasOModePower .NE. 0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "O-mode_power",  OModePowerID)                      )
     CALL CHECK( nf90_get_var(ncid,OModePowerID,     Ionogram%OPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "O-mode_noise",  OModeNoiseID)                      )
     CALL CHECK( nf90_get_var(ncid,OModeNoiseID,     Ionogram%ONoiseLevel(1:NFREQ)    ) )
ENDIF
!
! eXtraordinary Mode Power
CALL CHECK( nf90_inq_varid(ncid, "Has_X-mode_power", HasXModePowerID)                   )
  CALL CHECK( nf90_get_var(ncid,HasXModePowerID,     Ionogram%HasXmodePower)            )
IF (Ionogram%HasXModePower .NE. 0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "X-mode_power",  XModePowerID)                      )
      CALL CHECK( nf90_get_var(ncid,XModePowerID,    Ionogram%XPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "X-mode_noise",  XModeNoiseID)                      )
      CALL CHECK( nf90_get_var(ncid,XModeNoiseID,    Ionogram%XNoiseLevel(1:NFREQ)    ) )
ENDIF
!
! LoS Velocity Data
CALL CHECK( nf90_inq_varid(ncid, "Has_VLoS",       HasVLoSID)           )
   CALL CHECK( nf90_get_var(ncid,HasVLoSID,     Ionogram%HasVLoS) )
IF (Ionogram%HasVLoS.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "VLoS",        VLoSID)              )
     CALL CHECK( nf90_get_var(ncid,VloSID,    Ionogram%VLoS(1:NRANGE,1:NFREQ)) )
ENDIF
!

! Doppler Shift Data
CALL CHECK( nf90_inq_dimid(ncid,"NumDoppler",      Doppler_sizeID)            )
CALL CHECK( nf90_inquire_dimension(ncid,Doppler_sizeID, JUNK,Ionogram%NDOPP)  )
NDOPP=Ionogram%NDOPP 
CALL CHECK( nf90_inq_varid(ncid, "Has_Doppler",    HasDopplerID)              )
   CALL CHECK( nf90_get_var(ncid,HasDopplerID,     Ionogram%HasDoppler)       )
IF (Ionogram%HasDoppler.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "Doppler",     DopplerID)                 )
!xx   Need to implement allocatale arrays to handle this
!xx   CALL CHECK( nf90_get_var(ncid,XModePowerID,    Ionogram%Dopper(1:NDOPP,1:NRANGE,1:NFREQ)) )
ENDIF
!
! Stationary Phase Group Range or Precision Group Height
CALL CHECK( nf90_inq_varid(ncid, "Has_SPGR",      HasSPGRID)                  )
  CALL CHECK( nf90_get_var(ncid,HasSPGRID,        Ionogram%HasSPGR)           )
IF (Ionogram%HasSPGR.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "SPGR",       SPGRID)                     )
   CALL CHECK( nf90_get_var(ncid,SPGRID,    Ionogram%SPGR(1:NRANGE,1:NFREQ))  )
ENDIF
!
! Zenith Angle data
CALL CHECK( nf90_inq_varid(ncid, "Has_Zenith",    HasZenithID)                )
  CALL CHECK( nf90_get_var(ncid,HasZenithID,        Ionogram%HasZenith)       )
IF (Ionogram%HasZenith.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "zenith",     ZenithID)                   )
   CALL CHECK( nf90_get_var(ncid,ZenithID,    Ionogram%Zenith(1:NRANGE,1:NFREQ)) )
ENDIF
!
! Azimuth Angle
CALL CHECK( nf90_inq_varid(ncid, "Has_Azimuth",   HasAzimuthID)               )
   CALL CHECK( nf90_get_var(ncid,HasAzimuthID,    Ionogram%HasAzimuth)        )
IF (Ionogram%HasAzimuth.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "azimuth",    AzimuthID)                  )
   CALL CHECK( nf90_get_var(ncid,AzimuthID,    Ionogram%Azimuth(1:NRANGE,1:NFREQ)) )
ENDIF
!
! Coherence Arrays
CALL CHECK( nf90_inq_varid(ncid, "Has_Coherence",      HasCoherenceID)        )
  CALL CHECK( nf90_get_var(ncid,HasCoherenceID,        Ionogram%HasCoherence) )
IF (Ionogram%HasCoherence.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "baseline_pairs",  BaselinePairsID)       )
   CALL CHECK( nf90_inq_varid(ncid, "baseline_diffs",  BaselineDiffsID)       )
   CALL CHECK( nf90_inq_varid(ncid, "coherence",       CoherenceID)           )
!  Add coherence arrays here
ENDIF
!
! ----------------------------------------------------------------------------------------------------------------
!
! Global Attributes 
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "format_version", Ionogram%Format_Version ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "id", Ionogram%Identifier ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "naming_authority", Ionogram%NamingAuthority ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "Metadata_Link", Ionogram%MetadataLink ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "title", Ionogram%Title ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "station", Ionogram%Station ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "instrument", Ionogram%Instrument ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "cdm_data_type", Ionogram%CdmDataType ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "processing_level", Ionogram%ProcessingLevel ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "experiment", Ionogram%Experiment ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "date_created", Ionogram%DateCreated ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "date_modified", Ionogram%DateModified ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "date_issued", Ionogram%DateIssued ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "time_coverage_start", Ionogram%TimeCoverageStart ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "time_coverage_end", Ionogram%TimeCoverageEnd ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "time_coverage_duration", Ionogram%TimeCoverageDuration ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "time_coverage_resolution", Ionogram%TimeCoverageResolution ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lat_min", Ionogram%GeospatialLatMin ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lat_max", Ionogram%GeospatialLatMax ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lat_units", Ionogram%GeospatialLatUnits ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lat_resolution", Ionogram%GeospatialLatResolution ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lon_min", Ionogram%GeospatialLonMin ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lon_max", Ionogram%GeospatialLonMax ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lon_units", Ionogram%GeospatialLonUnits ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_lon_resolution", Ionogram%GeospatialLonResolution ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_vertical_min", Ionogram%GeospatialVerticalMin ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_vertical_max", Ionogram%GeospatialVerticalMax ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_vertical_units", Ionogram%GeospatialVerticalUnits ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_vertical_resolution", Ionogram%GeospatialVerticalResolution ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "geospatial_vertical_positive", Ionogram%GeospatialVerticalPositive ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "mode", Ionogram%Mode ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "magnetic_model", Ionogram%MagneticModel) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "creator_name", Ionogram%CreatorName ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "institution", Ionogram%Institution ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "creator_email", Ionogram%CreatorEmail ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "creator_url", Ionogram%CreatorUrl ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "publisher_name", Ionogram%PublisherName ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "publisher_email", Ionogram%PublisherEmail ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "publisher_url", Ionogram%PublisherUrl ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "license", Ionogram%License ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "project", Ionogram%Project ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "acknowledgment", Ionogram%Acknowledgment ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "contributor_name", Ionogram%ContributorName ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "contributor_role", Ionogram%ContributorRole ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "Conventions", Ionogram%Conventions ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "Metadata_Conventions", Ionogram%MetadataConventions) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "keywords", Ionogram%Keywords) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "keywords_vocabulary", Ionogram%KeywordsVocabulary ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "standard_name_vocabulary", Ionogram%StandardNameVocabulary ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "summary", Ionogram%Summary ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "comment", Ionogram%Comment ) )
CALL CHECK( nf90_get_att(ncid,NF90_GLOBAL, "history", Ionogram%History ) )
!
! Debug
!db call Dump_Attributes(Ionogram)

! Close the file.
CALL CHECK( nf90_close(ncid) )
!
!
RETURN
!
!
CONTAINS

SUBROUTINE CHECK(STATUS)
!
! Writes NetCDF errors to standard error 
!
INTEGER,INTENT(IN):: STATUS

IF ( STATUS.NE.NF90_NOERR) WRITE(0,*) "read_netCDF3 -- ERROR -- ", trim(nf90_strerror(STATUS))
RETURN
END SUBROUTINE CHECK 

END 
