PROGRAM read_sct
!
! Read the SCT from VIPIR Version 1.2 RIQ raw data files
! and write to stdout the values selected by the command line switches.
!
! Author: Terence Bullett,   CIRES
! 05 September 2011
! Linux using gfortran compiler
!
!-----------------------------------------------------------------------------------------
!
! Open files. NOTE:  This code REQUIRES the equivalent of MODE=BINARY or ACCESS=STREAM.
!             This is generally a F2003 standard, or unique to commercial compilers.
!             Your mileage will vary 
!-----------------------------------------------------------------------------------------
!
! License:
! This software is the intellectual property of the author.
! Generous terms of use are likely if you ask nicely.
! 
! "To give glory to God by exploring His universe with reverence"  
!
! Revision History
! 05Sep11 TWB  First version, date-time-stamp
!

IMPLICIT NONE
INCLUDE 'sct.f90'
INCLUDE 'pct.f90'

TYPE(SCTtype) SCT
TYPE(PCTtype) PCT
!
! Local Variables
CHARACTER(120)::INFILE,CLP 
LOGICAL:: INFOK,EOF,VERBOSE,DTS,SKIP 
INTEGER:: IU=1
INTEGER:: NA,K
REAL:: VERSION=2.06

! Functions
LOGICAL:: READSCT, READPCT 


! Read the command line for arguments
K = IARGC() 
!
SKIP=.FALSE.
VERBOSE=.FALSE.
DTS=.FALSE.
INFOK=.FALSE.

! Parse the command line.
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-v') THEN
      VERBOSE = .TRUE.
   ELSE IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: read_sct {options} filename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Read the VIPIR HF Radar Raw I/Q data file Sounding Configuration Table (SCT)'
      WRITE(*,*)'and output various values to stdout '
      WRITE(*,*)' '
      WRITE(*,*)'Options:'
      WRITE(*,*)' -dts Date-Time Stamp format UUUUU_YYYYDDDHHMMSS'
      WRITE(*,*)' Returns exit status as follows:'
      WRITE(*,*)'  0:  No Problems'
      WRITE(*,*)'  1:  read_sct error'
      WRITE(*,*)'  2:  File is not RIQ'
      WRITE(*,*)' -1:  Problems'
      RETURN
   ELSE IF (SKIP) THEN
      SKIP=.FALSE.
   ELSE IF (CLP.EQ.'-dts') THEN
      DTS=.TRUE.
   ELSE
!    This is the input file name
     INFILE=CLP
     INFOK=.TRUE.
   ENDIF
ENDDO 

! Filename must exist
IF (.NOT.INFOK) THEN
   WRITE(*,*) "read_sct -- ERROR: Input file must be specified"
   CALL EXIT(-1)
ENDIF
!
INFOK=.FALSE.
!     Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(0,*) ' dump_headers - error - Missing input file'
   CALL EXIT(-1)
ENDIF

CALL ZeroSCT(SCT)

OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
! Read the SCT
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
!
! Check the magic number for RIQ file
IF (SCT%magic.NE.1363759616) THEN
   CALL EXIT(2)
ENDIF 

CALL FixSCTstrings(SCT)
!
IF (DTS) THEN
!  Date-Time-stamp in format UUUUU_YYYYDDDHHMMSS
   WRITE(*,101) TRIM(SCT%station%ursi_id), SCT%start_year,SCT%start_daynumber,SCT%start_hour,SCT%start_minute,SCT%start_second
101 FORMAT (A5,'_',I4.4,I3.3,3I2.2)
ENDIF
!
END PROGRAM
