SUBROUTINE ReadConfig(ConfigFile,  Ionogram)
!
! Update the Ionogram structure using metatadata from a configuration file
! specified by the calling program.
!
! Terry Bullett
! University of Colorado Boulder
!
! 01 October 2010
!
! 07Sep11 TWB  Check to see if file exists.  
!              Added most global attributes as keywords.
!
! License: Public Domain
!
! "Knowledge, Sir, should be free to all"  -- Harry Mudd
!
!
IMPLICIT NONE
INCLUDE 'sct.f90'      ! SCT
INCLUDE 'ionogram.f90' ! Processed ionogram structure
!
! Passed Variables
CHARACTER(LEN=128),INTENT(IN):: ConfigFile     ! Full path/name of input file
TYPE(IonogramType),INTENT(INOUT) :: Ionogram
!
! Local Varables
INTEGER,PARAMETER:: IU=24
INTEGER:: IERR,POS
LOGICAL:: EOF
CHARACTER(LEN=256) :: Line
CHARACTER(LEN=128) :: Value
CHARACTER(LEN=64) :: Keyword
CHARACTER(LEN=1) :: HASH,EQUAL 

HASH='#'
EQUAL='='

! 
OPEN(UNIT=IU,FILE=ConfigFile, FORM='FORMATTED', ACCESS='SEQUENTIAL', STATUS='old', IOSTAT=IERR)

! Since this step is optional, all errors are non-fatal and are silently ignored.
IF (IERR.NE.0) RETURN

EOF=.FALSE.
DO WHILE (.NOT.EOF) 
   READ(IU,100,IOSTAT=IERR) Line
   IF (IERR.NE.0) THEN
      EOF=.TRUE.
      CYCLE
   ENDIF
!
!  Lines starting with a # are comments and ignored.
   IF(Keyword(1:1).EQ.HASH) CYCLE
!  Split the line across the = symbol
   POS=INDEX(Line, EQUAL)
   IF (POS.EQ.0) CYCLE   !  Invalid line
   Keyword=ADJUSTL((TRIM(Line(1 : POS-1))))
   Value=ADJUSTL(TRIM(Line(POS+1 : LEN_TRIM(Line))))
!  Strip strings to exclude values after #
   POS=INDEX(Keyword,HASH)
   IF (POS.GT.0) Keyword=Keyword(1 : POS-1)
   POS=INDEX(Value,HASH)
   IF (POS.GT.0) Value=Value(1 : POS-1)
!db   write(*,*) "DEBUG ", EOF, Keyword, Value 
!
!  Parse known Keyword--Value combinations
   SELECT CASE (TRIM(Keyword))
      CASE ("Title")
         Ionogram%Title=TRIM(Value)
      CASE ("Mode")
         Ionogram%Mode=TRIM(Value)
      CASE ("NamingAuthority")
         Ionogram%NamingAuthority=TRIM(Value)
      CASE ("MetadataLink")
         Ionogram%MetadataLink=TRIM(Value)
      CASE ("Station")
         Ionogram%Station=TRIM(Value)
      CASE ("Instrument") 
         Ionogram%Instrument=TRIM(Value)
      CASE ("CdmDataType")
         Ionogram%CdmDataType=TRIM(Value)
      CASE ("ProcessingLevel")
         Ionogram%ProcessingLevel=Trim(Value)
      CASE ("Experiment")
         Ionogram%Experiment=Trim(Value)
      CASE ("MagneticModel")
         Ionogram%MagneticModel=TRIM(Value)
      CASE ("CreatorName")
         Ionogram%CreatorName=Trim(Value)
      CASE ("Institution")
         Ionogram%Institution=TRIM(Value)
      CASE ("CreatorEmail")
         Ionogram%CreatorEmail=TRIM(Value)
      CASE ("CreatorUrl")
         Ionogram%CreatorUrl=TRIM(Value)
      CASE ("PublisherName")
         Ionogram%PublisherName=TRIM(Value)
      CASE ("PublisherEmail")
         Ionogram%PublisherEmail=TRIM(Value)
      CASE ("PublisherUrl")
         Ionogram%PublisherUrl=TRIM(Value)
      CASE ("License")
         Ionogram%License=TRIM(Value)
      CASE ("Project")
         Ionogram%Project=TRIM(Value)
      CASE ("Acknowledgment")
         Ionogram%Acknowledgment=TRIM(Value)
      CASE ("ContributorName")
         Ionogram%ContributorName=TRIM(Value)
      CASE ("ContributorRole")
         Ionogram%ContributorRole=TRIM(Value)
      CASE ("Conventions")
         Ionogram%Conventions=TRIM(Value)
      CASE ("MetadataConventions")
         Ionogram%MetadataConventions=TRIM(Value)
      CASE ("Keywords")
         Ionogram%Keywords=TRIM(Value)
      CASE ("KeywordsVocabulary")
         Ionogram%KeywordsVocabulary=TRIM(Value)
     CASE ("StandardNameVocabulary")
         Ionogram%StandardNameVocabulary=TRIM(Value)
     CASE ("Summary")
         Ionogram%Summary=TRIM(Value)
     CASE ("Comment")
         Ionogram%Comment=TRIM(Value)
     CASE ("History")
         Ionogram%History=TRIM(Value)
     CASE DEFAULT
         WRITE(*,*) " ReadConfig - Unknown entry - Keyword=",TRIM(Keyword), ' Value=',TRIM(Value)
         CONTINUE

   END SELECT
END DO   ! Each line in the file    


!db write(*,*) 'Mode = ', TRIM(Ionogram%Mode)

CLOSE(UNIT=IU)

RETURN

100 FORMAT (A128)
END

! Other global attributes not yet programmed into this code
!
!		:date_modified = "2011-09-07T15:19:39Z" ;
!		:date_issued = "2011-09-07T15:19:39Z" ;
!		:time_coverage_start = "2010-02-10T22:10:33Z" ;
!		:time_coverage_end = "2010-02-10T22:14:46Z" ;
!		:time_coverage_duration = "TM04S13" ;
!		:time_coverage_resolution = "none" ;
!		:geospatial_lat_min = "    39.892" ;
!		:geospatial_lat_max = "    40.092" ;
!		:geospatial_lat_units = "degrees" ;
!		:geospatial_lat_resolution = "none" ;
!		:geospatial_lon_min = "  -105.369" ;
!		:geospatial_lon_max = "  -105.169" ;
!		:geospatial_lon_units = "degrees" ;
!		:geospatial_lon_resolution = "none" ;
!		:geospatial_vertical_min = "20" ;
!		:geospatial_vertical_max = "500" ;
!		:geospatial_vertical_units = "kilometers" ;
!		:geospatial_vertical_resolution = "variable" ;
!		:geospatial_vertical_positive = "up" ;
